@@ -87,7 +87,7 @@ class GalleryTask(DeeplabelBase):
         return detections
 
     def insert_detections(
-        self, detections: List[gallery_detections.Detection], chunk_size: int = 500
+        self, detections: List[gallery_detections.Detection], chunk_size: int = 500, lean:bool = False, type:str = "IMAGE_BOUNDING_BOX"
     ) -> None:
         assert bool(
             detections
@@ -107,7 +107,7 @@ class GalleryTask(DeeplabelBase):
             count += len(data)
             self.client.post(
                 "/image-detections",
-                {"batch": True, "data": data, "galleryTaskId": self.gallery_task_id},
+                {"batch": True, "lean": lean, "type":type, "data": data, "galleryTaskId": self.gallery_task_id},
             )
         logger.debug(
             f"Completed pushing {len(detections)} detections for galleryTaskId: {self.gallery_task_id}"
