@@ -2,10 +2,11 @@
 Module to get videos data
 """
 from enum import Enum
+import time
 from typing import List, Optional, Dict, Any
+from urllib.parse import urlparse, parse_qs
 from pydantic import Field
 from deeplabel.basemodel import DeeplabelBase, MixinConfig
-from pydantic import validator
 import deeplabel.label.videos.frames
 import deeplabel.label.videos.detections
 import deeplabel.label.videos.video_tasks
@@ -33,7 +34,7 @@ class _VideoFormat(MixinConfig):
 
 
 class _VideoUrl(MixinConfig):
-    source: Optional[_VideoFormat]
+    source: _VideoFormat
     res360: Optional[_VideoFormat] = Field(None, alias="360P")
     res480: Optional[_VideoFormat] = Field(None, alias="480P")
     res720: Optional[_VideoFormat] = Field(None, alias="720P")
@@ -89,9 +90,6 @@ class Video(DeeplabelBase):
     project_id: str
     input_url: str
     video_urls: Optional[_VideoUrl]
-    # videoUrls.source.url if exists else videoUrl is used for legacy support.
-    # In case both are absent, use input_url
-    video_url: Optional[str]
     thumbnail_url: Optional[str]
     status: _LabelVideoStatus
     extraction_points: List[ExtractionPoint]
@@ -124,27 +122,39 @@ class Video(DeeplabelBase):
         # return cls.from_video_id(video_id, client)
         return video_id
 
-    @validator("video_url", always=True)
-    def validate_url(cls, value, values: Dict[str, Any]):  # type: ignore
-        """
-        Validate that either video_url or video_urls.source.url exists
-        Since videoUrl can't be updated anymore, since it's deprecated,
-        first check if videoUrls.source.url exists, and use that,
-        else look for videoUrl
-        Refer https://github.com/samuelcolvin/pydantic/issues/832#issuecomment-534896056
-        """
-        # video_urls.source.url
-        try:
-            source_url: str = values.get("video_urls", {}).source.url  # type: ignore
-        except:
-            # should have either of the two
-            # If video_url key is empty
-            if isinstance(value, str):
-                return value
-            else:
-                return values.get("input_url", "")
-        # set video_url = video_urls.source.url
-        return source_url
+    @property
+    def video_url(self) -> str:
+        """This API generates a new presigned video url incase the old one has expired"""
+        if self.video_urls is not None:
+            url = self.video_urls.source.url
+        else:
+            url = self.extra.get('video_url', '')
+
+        parsed = urlparse(url)
+        query = parse_qs(parsed.query)
+        expiry_key:Optional[str] = None
+        for key in query.keys():
+            if key.lower().startswith('expire'):
+                expiry_key = key
+                break
+        # if expiry is more that now + 30 sec. don't update
+        if expiry_key and int(query[expiry_key][0]) > int(time.time()) + 30:
+            return url
+        # if presigned url that is expired
+        # or is about to expire or
+        # doesn't has a video_url
+        # or head request fails
+        elif expiry_key or (not url or self.client.session.head(url).status_code != 200):
+            video = self._generate_video_url(self.video_id, self.client)
+            self.video_urls = video.video_urls
+            return self.video_urls.source.url #type: ignore # since this api will always return a valid source.url
+        return url
+
+    @staticmethod
+    def _generate_video_url(video_id:str, client:"deeplabel.client.BaseClient"):
+        resp = client.get('/projects/videos/video-url', params={"videoId":video_id})
+        video = Video(**resp.json()['data']['videos'][0])
+        return video
 
     @classmethod
     def from_search_params(
@@ -251,7 +261,7 @@ class Video(DeeplabelBase):
             request_detections:List[Dict[str, Any]] = []
             for detection in detections[chunk_size * i : chunk_size * (i + 1)]:
                 det = {
-                    "labelId": detection.label_id.label_id,
+                    "labelId": detection.label.label_id,
                     "type": detection.type.value,
                     "frameId": detection.frame_id
                 }
